'use strict';

const Service = require('egg').Service;
const dayjs = require('dayjs');
class CurrentMonthRankService extends Service {

  async infodept(query) {
    const { ctx } = this;
    query.status = '3';
    let deptGroup = '';
    if (query.dept1) {
      deptGroup = '$dept2';
    }
    if (query.dept2) {
      deptGroup = '$dept3';
    }
    if (query.dept3) {
      deptGroup = '$dept4';
    }
    const resultList = await ctx.model.InfoModel.aggregate([
      { $match: ctx.alterDeptId(query) },
      { $project: {
        _id: 1, dept1: 1, dept2: 1, dept3: 1, dept4: 1, dept5: 1,
        // 这里需要加8小时,数据库里面的时间是ISO类型
        infoTypeOfYear: { $sum: {
          $cond: [{ $eq: [
            { $dateToString: { format: '%Y-%m', date: { $add: [ '$time', 1000 * 60 * 60 * 8 ] } } },
            { $dateToString: { format: '%Y-%m', date: { $add: [ new Date(), 1000 * 60 * 60 * 8 ] } } },
          ] }, 1, 0 ],
        } },
      },
      },
      { $match: { infoTypeOfYear: 1 } },
      { $group: {
        _id: deptGroup,
        count: { $sum: 1 },
      },
      },
      { $lookup: { from: 'sys_dept', localField: '_id', foreignField: '_id', as: 'deptInfo' } },
      { $unwind: { path: '$deptInfo', preserveNullAndEmptyArrays: true } },
      { $project: {
        _id: 1, label: '$deptInfo.name', value: '$count',
      },
      },
      { $sort: { value: -1 } },
      {
        $skip: 0,
      },
      {
        $limit: 5,
      },
    ]);
    const resultArr = [];
    try {
      resultList.forEach(item =>
        resultArr.push({ label: item.label, value: item.value })
      );
    } catch (e) {
      ctx.logger.error('catch--------error', e);
      ctx.error(e);
    } finally {
      return resultArr;
    }
  }

  async infoperson(query) {
    const { ctx } = this;
    query.status = '3';
    const resultList = await ctx.model.InfoModel.aggregate([
      { $match: ctx.alterDeptId(query) },
      { $project: {
        _id: 1, userid: 1,
        // 这里需要加8小时,数据库里面的时间是ISO类型
        infoTypeOfYear: { $sum: {
          $cond: [{ $eq: [
            { $dateToString: { format: '%Y-%m', date: { $add: [ '$time', 1000 * 60 * 60 * 8 ] } } },
            { $dateToString: { format: '%Y-%m', date: { $add: [ new Date(), 1000 * 60 * 60 * 8 ] } } },
          ] }, 1, 0 ],
        } },
      },
      },
      { $match: { infoTypeOfYear: 1 } },
      { $group: {
        _id: '$userid',
        count: { $sum: 1 },
      },
      },
      { $lookup: { from: 'sys_user', localField: '_id', foreignField: '_id', as: 'userInfo' } },
      { $unwind: { path: '$userInfo', preserveNullAndEmptyArrays: true } },
      { $project: {
        _id: 1, label: '$userInfo.userName', value: '$count',
      },
      },
      { $sort: { value: -1 } },
      {
        $skip: 0,
      },
      {
        $limit: 5,
      },
    ]);
    const resultArr = [];
    try {
      resultList.forEach(item =>
        resultArr.push({ label: item.label, value: item.value })
      );
    } catch (e) {
      ctx.logger.error('catch--------error', e);
      ctx.error(e);
    } finally {
      return resultArr;
    }
  }

  async visitdept(query) {
    const { ctx } = this;
    let deptGroup = '';
    if (query.dept1) {
      deptGroup = '$dept2';
    }
    if (query.dept2) {
      deptGroup = '$dept3';
    }
    if (query.dept3) {
      deptGroup = '$dept4';
    }
    const resultList = await ctx.model.VisitModel.aggregate([
      { $match: ctx.alterDeptId(query) },
      { $project: {
        _id: 1, dept1: 1, dept2: 1, dept3: 1, dept4: 1,
        // 这里需要加8小时,数据库里面的时间是ISO类型
        visitTypeOfYear: { $sum: {
          $cond: [{ $eq: [
            { $dateToString: { format: '%Y-%m', date: { $add: [ '$visitTime', 1000 * 60 * 60 * 8 ] } } },
            { $dateToString: { format: '%Y-%m', date: { $add: [ new Date(), 1000 * 60 * 60 * 8 ] } } },
          ] }, 1, 0 ],
        } },
      },
      },
      { $match: { visitTypeOfYear: 1 } },
      { $group: {
        _id: deptGroup,
        count: { $sum: 1 },
      },
      },
      { $lookup: { from: 'sys_dept', localField: '_id', foreignField: '_id', as: 'deptInfo' } },
      { $unwind: { path: '$deptInfo', preserveNullAndEmptyArrays: true } },
      { $project: {
        _id: 1, label: '$deptInfo.name', value: '$count',
      },
      },
      { $sort: { value: -1 } },
      {
        $skip: 0,
      },
      {
        $limit: 5,
      },
    ]);
    const resultArr = [];
    try {
      resultList.forEach(item =>
        resultArr.push({ label: item.label, value: item.value })
      );
    } catch (e) {
      ctx.logger.error('catch--------error', e);
      ctx.error(e);
    } finally {
      return resultArr;
    }
  }

  async visitperson(query) {
    const { ctx } = this;

    const resultList = await ctx.model.VisitModel.aggregate([
      { $match: ctx.alterDeptId(query) },
      { $project: {
        _id: 1, dept1: 1, dept2: 1, dept3: 1, dept4: 1, userid: 1,
        // 这里需要加8小时,数据库里面的时间是ISO类型
        visitTypeOfYear: { $sum: {
          $cond: [{ $eq: [
            { $dateToString: { format: '%Y-%m', date: { $add: [ '$visitTime', 1000 * 60 * 60 * 8 ] } } },
            { $dateToString: { format: '%Y-%m', date: { $add: [ new Date(), 1000 * 60 * 60 * 8 ] } } },
          ] }, 1, 0 ],
        } },
      },
      },
      { $match: { visitTypeOfYear: 1 } },
      { $group: {
        _id: '$userid',
        count: { $sum: 1 },
      },
      },
      { $lookup: { from: 'sys_user', localField: '_id', foreignField: '_id', as: 'userInfo' } },
      { $unwind: { path: '$userInfo', preserveNullAndEmptyArrays: true } },
      { $project: {
        _id: 1, label: '$userInfo.userName', value: '$count',
      },
      },
      { $sort: { value: -1 } },
      {
        $skip: 0,
      },
      {
        $limit: 5,
      },
    ]);
    const resultArr = [];
    try {
      resultList.forEach(item =>
        resultArr.push({ label: item.label, value: item.value })
      );
    } catch (e) {
      ctx.logger.error('catch--------error', e);
      ctx.error(e);
    } finally {
      return resultArr;
    }
  }

  async visitnumday(query) {

    const { ctx } = this;

    const lastWeekStartTime = dayjs().subtract(1, 'week').startOf('week').$d;
    const nowWeekStartTime = dayjs().startOf('week').$d;
    query.visitTime = { $gte: lastWeekStartTime, $lt: nowWeekStartTime };

    const resultList = await ctx.model.VisitModel.aggregate([
      { $match: ctx.alterDeptId(query) },
      {
        $group: {
          _id: null,
          count: { $sum: 1 },
        },
      },
      {
        $project: { _id: 0, count: 1 },
      },
    ]);

    const resultCollection = { visitnum: resultList[0] ? resultList[0].count : 0 };
    return resultCollection;
  }

  async selectInfoMessage(query) {
    const { ctx } = this;
    const resultList = await ctx.model.VisitModel.aggregate([
      { $match: ctx.alterDeptId(query) },
      { $sort: { visitTime: -1 } },
      {
        $skip: 0,
      },
      {
        $limit: 100,
      },
      { $lookup: { from: 'info', localField: 'infoId', foreignField: '_id', as: 'infoMsg' } },
      { $unwind: { path: '$infoMsg', preserveNullAndEmptyArrays: true } },
      { $lookup: { from: 'sys_dept', localField: 'dept2', foreignField: '_id', as: 'dept2Info' } },
      { $unwind: { path: '$dept2Info', preserveNullAndEmptyArrays: true } },
      { $lookup: { from: 'sys_dept', localField: 'dept3', foreignField: '_id', as: 'dept3Info' } },
      { $unwind: { path: '$dept3Info', preserveNullAndEmptyArrays: true } },
      { $project: {
        visitTimeYMD: { $dateToString: { format: '%Y-%m-%d', date: { $add: [ '$visitTime', 1000 * 60 * 60 * 8 ] } } },
        visitTimeHI: { $dateToString: { format: '%H:%M', date: { $add: [ '$visitTime', 1000 * 60 * 60 * 8 ] } } },
        visitMsg: { $concat: [
          '$dept2Info.name', '$dept3Info.name',
          { $arrayElemAt: [ '$infoMsg.oldType', 0 ] },
        ] },
        oldInfo: '$oldInfo',
      },
      },
    ]);
    const resultArr = [];
    try {
      resultList.forEach(item =>
        resultArr.push({ date: item.visitTimeYMD, address: item.visitMsg, oldInfo: item.oldInfo, time: item.visitTimeHI })
      );
    } catch (e) {
      ctx.logger.error('catch--------error', e);
      ctx.error(e);
    } finally {
      return resultArr;
    }
  }

}
module.exports = CurrentMonthRankService;
